/*
 * attr.c
 *
 * Copyright 2022 Jonathan Schöbel <jonathan@Ubermos-2019>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 *
 */


#include <stdlib.h>

#include "macro.h"
#include "log.h"
#include "status.h"

#include "attr.h"


struct SH_Attr
{
	/*@only@*/ char * name;
	/*@null@*/ /*@only@*/ char * value;
};


/*@null@*/
/*@only@*/
struct SH_Attr *
SH_Attr_new (const char * name,
             /*@null@*/ const char * value,
             /*@null@*/ /*@out@*/ struct SH_Status * status)
	/*@globals fileSystem@*/
	/*@modifies fileSystem@*/
	/*@modifies status@*/
{
	struct SH_Attr * attr;
	attr = malloc (sizeof (struct SH_Attr));

	if (attr == NULL)
	{
		set_status (status, E_ALLOC, 4, "malloc failed");
		return NULL;
	}

	attr->name = strdup (name);
	if (NULL == attr->name)
	{
		set_status (status, E_ALLOC, 2, "strdup failed");
		free (attr);
		return NULL;
	}

	if (NULL == value)
	{
		attr->value = NULL;
	}
	else
	{
		attr->value = strdup (value);
		if (NULL == attr->value)
		{
			set_status (status, E_ALLOC, 2,
			            "strdup failed");
			free (attr->name);
			free (attr);
			return NULL;
		}
	}

	set_success (status);
	return attr;
}

void
SH_Attr_free (/*@only@*/ struct SH_Attr * attr)
	/*@modifies attr->name@*/
	/*@modifies attr->value@*/
	/*@modifies attr@*/
	/*@releases attr@*/
{
	free (attr->name);
	if (NULL != attr->value) free (attr->value);
	free (attr);

	return;
}
